#ifndef _AFE_Configuration_T_h
#define _AFE_Configuration_T_h

#define AFE_FIRMWARE_VERSION "3.8.0.B0"
#define AFE_FIRMWARE_TYPE 5

/**
 * @brief Hardware
 * 
 */

/* For all, E1, E2 */
#define AFE_CONFIG_HARDWARE_SWITCH
#define AFE_CONFIG_HARDWARE_LED
#define AFE_CONFIG_FUNCTIONALITY_ADC


/* For ESP32 so E1, E2 */
#ifdef AFE_ESP32

#ifdef AFE_T5_CUSTOM_E1 /* Hardware: E1 */
#define AFE_CONFIG_HARDWARE_PN532_SENSOR
#define AFE_CONFIG_HARDWARE_CLED
#define AFE_CONFIG_HARDWARE_CLED_ACCESS_CONTROL_EFFECT
#define AFE_CONFIG_HARDWARE_CLED_LIGHT_CONTROLLED_EFFECT

#elif AFE_T5_CUSTOM_E2 /* Hardware: E2 */
#define AFE_CONFIG_HARDWARE_DS18B20
#endif 

#endif // End of custom versions for ESP32

/* For 4MB, incl E1, E2 */
#ifdef AFE_ESP_FLASH_4MB 

#define AFE_CONFIG_HARDWARE_I2C
#define AFE_CONFIG_HARDWARE_BMEX80
#define AFE_CONFIG_HARDWARE_BH1750
#define AFE_CONFIG_HARDWARE_TSL2561
#define AFE_CONFIG_HARDWARE_BINARY_SENSOR
// #define AFE_CONFIG_HARDWARE_DHT Not suppoeted yet by 3.7

#endif // AFE_ESP_FLASH_4M


/**
 * @brief Max number of hardware items per AFE version
 * 
 */

/* For all, E1, E2 */

#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_CONTACTRONS 4
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_GATES 2


#ifdef AFE_T5_CUSTOM_E1 /* For version E1 */
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_PN532 1
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_MIFARE_CARDS 10
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_CLED_STRIPS 2
#endif


#ifdef AFE_T5_CUSTOM_E2 /* For version E2 */

#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_RELAYS 8
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_SWITCHES 9
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_LEDS 9
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_DS18B20 1

#else /* For none E2 */

#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_RELAYS 2
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_SWITCHES 3
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_LEDS 3

#endif


// Sensors only available in 4MB Flash
#ifdef AFE_ESP_FLASH_4MB

#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_BMEX80 1
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_BH1750 1
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_TSL2561 1
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_BINARY_SENSORS 4
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_DHT 1

/* Number of I2C depending on ESP type*/
#ifdef AFE_ESP32
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_I2C 2
#else // ESP8266 
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_I2C 1
#endif // ESP32 || ESP8266

#endif // end 4mb



/* Max number of hardware items per specyfic hardware device */
#if defined(AFE_DEVICE_iECS_GATE_DRIVERv2)
#define AFE_CONFIG_HARDWARE_NUMBER_OF_RELAYS 2
#define AFE_CONFIG_HARDWARE_NUMBER_OF_SWITCHES 2
#define AFE_CONFIG_HARDWARE_NUMBER_OF_LEDS 1
#define AFE_CONFIG_HARDWARE_NUMBER_OF_CONTACTRONS 3
#define AFE_CONFIG_HARDWARE_NUMBER_OF_GATES 2
#elif defined(AFE_DEVICE_iECS_GATE_DRIVERv3)
#define AFE_CONFIG_HARDWARE_NUMBER_OF_RELAYS 2
#define AFE_CONFIG_HARDWARE_NUMBER_OF_SWITCHES 1
#define AFE_CONFIG_HARDWARE_NUMBER_OF_LEDS 1
#define AFE_CONFIG_HARDWARE_NUMBER_OF_CONTACTRONS 4
#define AFE_CONFIG_HARDWARE_NUMBER_OF_GATES 2
#elif defined(AFE_T5_CUSTOM_E2)
#define AFE_CONFIG_HARDWARE_NUMBER_OF_RELAYS AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_RELAYS
#define AFE_CONFIG_HARDWARE_NUMBER_OF_SWITCHES AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_SWITCHES
#define AFE_CONFIG_HARDWARE_NUMBER_OF_LEDS AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_LEDS
#define AFE_CONFIG_HARDWARE_NUMBER_OF_CONTACTRONS AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_CONTACTRONS
#define AFE_CONFIG_HARDWARE_NUMBER_OF_GATES AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_GATES
#define AFE_CONFIG_HARDWARE_NUMBER_OF_DS18B20 AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_DS18B20
#else /* Generic version */
#define AFE_CONFIG_HARDWARE_NUMBER_OF_RELAYS 2
#define AFE_CONFIG_HARDWARE_NUMBER_OF_SWITCHES 3
#define AFE_CONFIG_HARDWARE_NUMBER_OF_LEDS 3
#define AFE_CONFIG_HARDWARE_NUMBER_OF_CONTACTRONS 4
#define AFE_CONFIG_HARDWARE_NUMBER_OF_GATES 2
#endif

#ifdef AFE_ESP_FLASH_4MB
#define AFE_CONFIG_HARDWARE_NUMBER_OF_BMEX80 AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_BMEX80
#define AFE_CONFIG_HARDWARE_NUMBER_OF_BH1750 AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_BH1750
#define AFE_CONFIG_HARDWARE_NUMBER_OF_TSL2561 AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_TSL2561
#define AFE_CONFIG_HARDWARE_NUMBER_OF_I2C AFE_CONFIG_HARDWARE_MAX_NUMBER_I2C
#define AFE_CONFIG_HARDWARE_NUMBER_OF_BINARY_SENSORS AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_BINARY_SENSORS
#define AFE_CONFIG_HARDWARE_NUMBER_OF_DHT AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_DHT
#endif


#ifdef AFE_T5_CUSTOM_E1
#define AFE_CONFIG_HARDWARE_NUMBER_OF_CLED_STRIPS AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_CLED_STRIPS
#define AFE_CONFIG_HARDWARE_NUMBER_OF_PN532_SENSORS AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_PN532
#define AFE_CONFIG_HARDWARE_NUMBER_OF_MIFARE_CARDS AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_MIFARE_CARDS
#define AFE_CONFIG_HARDWARE_NUMBER_OF_MIFARE_CARDS_NONE_PRO_VERSION 4
#endif // AFE_T5_CUSTOM_E1


/* Default values for hardware items per specyfic hardware device */
#if defined(AFE_DEVICE_iECS_GATE_DRIVERv2) || defined(AFE_DEVICE_iECS_GATE_DRIVERv3)
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_RELAYS 1
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_SWITCHES 1
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_LEDS 1
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_CONTACTRONS 1
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_GATES 1
#else
/* Generic version */
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_RELAYS 0
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_SWITCHES 0
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_LEDS 1
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_CONTACTRONS 0
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_GATES 0
#endif


#ifdef AFE_ESP_FLASH_4MB
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_BMEX80 0
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_BH1750 0
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_TSL2561 0
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_I2C 0
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_BINARY_SENSORS 0
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_DHT 0
#endif  


#ifdef AFE_T5_CUSTOM_E1
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_CLED_STRIPS 0
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_PN532_SENSORS 0
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_MIFARE_CARDS 0
#endif // AFE_T5_CUSTOM_E1


#ifdef AFE_T5_CUSTOM_E2
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_DS18B20 0
#endif // AFE_T5_CUSTOM_E1


/* Functionalities */
#define AFE_CONFIG_FUNCTIONALITY_RELAY
#define AFE_CONFIG_FUNCTIONALITY_RELAY_AUTOONOFF
#define AFE_CONFIG_FUNCTIONALITY_RELAY_CONTROL_AUTOONOFF_TIME
#define AFE_CONFIG_FUNCTIONALITY_GATE
#define AFE_CONFIG_FUNCTIONALITY_ADC

#ifdef AFE_CONFIG_FUNCTIONALITY_ADC
#ifdef AFE_ESP32
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_ADCS AFE_NUMBER_OF_ADC_GPIOS
#else // ESP8266
#define AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_ADCS 1
#endif // ESP32/ESP8266

#define AFE_CONFIG_HARDWARE_NUMBER_OF_ADCS AFE_CONFIG_HARDWARE_MAX_NUMBER_OF_ADCS
#define AFE_CONFIG_HARDWARE_DEFAULT_NUMBER_OF_ADCS 0

#endif // AFE_CONFIG_FUNCTIONALITY_ADC
#endif // _AFE_Configuration_T_h